@@ -24,6 +24,14 @@ getTradingDates <- function(end, start=NULL, obs=NULL, calendar=holidayNYSE)
     start <- as.Date(end) - shimmed + 1
     dates <- timeSequence(from=start, length.out=shimmed)
     dates <- dates[isBizday(dates, holidays=calendar())]
+    if (length(dates) < obs)
+    {
+      # It turns out that there are a lot of holidays so add a few more days
+      gap <- (2 + obs - length(dates)) * 2
+      start.1 <- as.Date(dates[1] - shimmed)
+      dates.1 <- timeSequence(from=start, length.out=gap)
+      dates <- c(dates.1[isBizday(dates.1, holidays=calendar())], dates)
+    }
 
     inf <- anylength(dates) - obs + 1
     sup <- anylength(dates)
@@ -122,7 +130,7 @@ fractal.uniform <- function(seed, patterns, count=NULL, epochs=NULL,
     pattern.legs <- nrow(patterns[[1]])-1
     epochs <- floor(log(count / seed.legs, base=pattern.legs)) + 1
     #cat("seed.legs:",seed.legs,"; pattern.legs:",pattern.legs,"\n")
-    logger(DEBUG, sprintf("Set epochs to",epochs,"\n"))
+    logger(DEBUG, sprintf("Set epochs to %s",epochs))
   }
   if (is.na(epochs) | is.null(epochs)) stop("Unable to calculate epochs")
 
@@ -173,18 +181,32 @@ next.seeds <- function(old.seed, new.seed, pattern, idx, epoch)
   return(list(this.seed=old.seed, next.seed=new.seed))
 }
 
-fractal.random <- function(seed, patterns, count)
+fractal.random <- function(seed, patterns, count=NULL, epochs=NULL, 
+  origin='1970-01-01', date.fun=as.Date, only=NULL)
 {
-  if (! 'list' %in% class(patterns))
-  {
-    patterns <- list(pattern.1=patterns)
-  }
+  if (! 'list' %in% class(patterns)) patterns <- list(pattern.1=patterns)
 
-  for (dummy in 1:count)
+  # Calculate count based on size of seed and patterns
+  if (! is.null(epochs))
+  {
+    use.count <- FALSE
+    count <- (length(seed) - 1) * (length(patterns[[1]])-1)^epochs
+  }
+  else if (! is.null(count))
+  {
+    use.count <- TRUE
+    seed.legs <- nrow(seed)-1
+    pattern.legs <- nrow(patterns[[1]])-1
+    epochs <- floor(log(count / seed.legs, base=pattern.legs)) + 1
+    #cat("seed.legs:",seed.legs,"; pattern.legs:",pattern.legs,"\n")
+    logger(DEBUG, sprintf("Set epochs to %s",epochs))
+  }
+  if (is.na(epochs) | is.null(epochs)) stop("Unable to calculate epochs")
+
+  for (dummy in 1:epochs)
   {
     pattern <- patterns[[sample(length(patterns),1)]]
     idx <- sample((nrow(seed)-1), 1) + 1
-    #cat("Got index",idx,"\n")
 
     x.delta <- seed[idx,1] - seed[idx-1,1]
     y.delta <- seed[idx,2] - seed[idx-1,2]
@@ -195,10 +217,11 @@ fractal.random <- function(seed, patterns, count)
       matrix(rep(start, nrow(pattern)), ncol=2, byrow=TRUE)
     if (idx <= 2)
     {
-      next.seed <- rbind(segment, seed[idx:nrow(seed),])
+      next.seed <- rbind(segment, seed[(idx+1):nrow(seed),])
     }
     else if (idx == nrow(seed))
     {
+      # Last row
       next.seed <- rbind(seed[1:(idx-2),], segment)
     }
     else
@@ -207,7 +230,11 @@ fractal.random <- function(seed, patterns, count)
     }
     seed <- next.seed[order(next.seed[,1]),]
   }
-  return(unique(seed))
+  # TODO: This may cause problems related to binary order sign
+  seed <- (unique(seed))
+  seed <- xts(seed[,2], order.by=date.fun(seed[,1], origin=origin))
+  if (use.count) seed <- tail(seed, count)
+  return(seed)
 }
 
 
